"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDeepQuery = exports.useDeep = exports.useAuthNode = exports.GUEST = exports.WHOISME = exports.JWT = exports.DeepClient = exports.parseJwt = exports.GLOBAL_ID_INCLUDE_TO_CURRENT_FROM = exports.GLOBAL_ID_INCLUDE_FROM_CURRENT_TO = exports.GLOBAL_ID_INCLUDE_CURRENT_TO = exports.GLOBAL_ID_INCLUDE_CURRENT_FROM = exports.GLOBAL_ID_INCLUDE_TO_CURRENT = exports.GLOBAL_ID_INCLUDE_FROM_CURRENT = exports.GLOBAL_ID_INCLUDE_OUT = exports.GLOBAL_ID_INCLUDE_IN = exports.GLOBAL_ID_SELECTOR_FILTER = exports.GLOBAL_ID_ALLOW_ADMIN = exports.GLOBAL_ID_JOIN = exports.GLOBAL_ID_HANDLE_UPDATE = exports.GLOBAL_ID_PACKAGE_VERSION = exports.GLOBAL_ID_PACKAGE_ACTIVE = exports.GLOBAL_ID_PACKAGE_NAMESPACE = exports.GLOBAL_ID_CONTAIN_TREE = exports.GLOBAL_ID_INCLUDE_NODE = exports.GLOBAL_ID_INCLUDE_UP = exports.GLOBAL_ID_INCLUDE_DOWN = exports.GLOBAL_ID_TREE = exports.GLOBAL_ID_REJECTED = exports.GLOBAL_ID_RESOLVED = exports.GLOBAL_ID_THEN = exports.GLOBAL_ID_PROMISE = exports.GLOBAL_ID_ANY = exports.GLOBAL_ID_OBJECT = exports.GLOBAL_ID_NUMBER = exports.GLOBAL_ID_STRING = exports.GLOBAL_ID_CONTAIN = exports.GLOBAL_ID_PACKAGE = exports.DENIED_IDS = exports.ALLOWED_IDS = void 0;
const atob_1 = require("atob");
const client_1 = require("@apollo/client");
const client_2 = require("@deep-foundation/hasura/client");
const local_1 = require("@deep-foundation/store/local");
const react_1 = require("react");
const gql_1 = require("./gql");
const minilinks_1 = require("./minilinks");
const promise_1 = require("./promise");
const react_token_1 = require("./react-token");
const reserve_1 = require("./reserve");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:client');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
exports.ALLOWED_IDS = [5];
exports.DENIED_IDS = [0, 10, 11, 12, 13];
exports.GLOBAL_ID_PACKAGE = 2;
exports.GLOBAL_ID_CONTAIN = 3;
exports.GLOBAL_ID_STRING = 5;
exports.GLOBAL_ID_NUMBER = 6;
exports.GLOBAL_ID_OBJECT = 7;
exports.GLOBAL_ID_ANY = 8;
exports.GLOBAL_ID_PROMISE = 9;
exports.GLOBAL_ID_THEN = 10;
exports.GLOBAL_ID_RESOLVED = 11;
exports.GLOBAL_ID_REJECTED = 12;
exports.GLOBAL_ID_TREE = 36;
exports.GLOBAL_ID_INCLUDE_DOWN = 37;
exports.GLOBAL_ID_INCLUDE_UP = 38;
exports.GLOBAL_ID_INCLUDE_NODE = 39;
exports.GLOBAL_ID_CONTAIN_TREE = 40;
exports.GLOBAL_ID_PACKAGE_NAMESPACE = 43;
exports.GLOBAL_ID_PACKAGE_ACTIVE = 45;
exports.GLOBAL_ID_PACKAGE_VERSION = 46;
exports.GLOBAL_ID_HANDLE_UPDATE = 50;
exports.GLOBAL_ID_JOIN = 66;
exports.GLOBAL_ID_ALLOW_ADMIN = 71;
exports.GLOBAL_ID_SELECTOR_FILTER = 75;
exports.GLOBAL_ID_INCLUDE_IN = 113;
exports.GLOBAL_ID_INCLUDE_OUT = 114;
exports.GLOBAL_ID_INCLUDE_FROM_CURRENT = 115;
exports.GLOBAL_ID_INCLUDE_TO_CURRENT = 116;
exports.GLOBAL_ID_INCLUDE_CURRENT_FROM = 117;
exports.GLOBAL_ID_INCLUDE_CURRENT_TO = 118;
exports.GLOBAL_ID_INCLUDE_FROM_CURRENT_TO = 119;
exports.GLOBAL_ID_INCLUDE_TO_CURRENT_FROM = 120;
const _ids = {
    '@deep-foundation/core': {
        'Contain': exports.GLOBAL_ID_CONTAIN,
        'Join': exports.GLOBAL_ID_JOIN,
        'containTree': exports.GLOBAL_ID_CONTAIN_TREE,
        'Package': exports.GLOBAL_ID_PACKAGE,
        'PackageActive': exports.GLOBAL_ID_PACKAGE_ACTIVE,
        'PackageVersion': exports.GLOBAL_ID_PACKAGE_VERSION,
        'PackageNamespace': exports.GLOBAL_ID_PACKAGE_NAMESPACE,
        'Promise': exports.GLOBAL_ID_PROMISE,
        'Then': exports.GLOBAL_ID_THEN,
        'Resolved': exports.GLOBAL_ID_RESOLVED,
        'Rejected': exports.GLOBAL_ID_REJECTED,
        'HandleUpdate': exports.GLOBAL_ID_HANDLE_UPDATE,
        'AllowAdmin': exports.GLOBAL_ID_ALLOW_ADMIN,
    },
};
function parseJwt(token) {
    var base64Url = token.split('.')[1];
    var base64 = base64Url.replace(/-/g, '+').replace(/_/g, '/');
    var jsonPayload = decodeURIComponent((0, atob_1.atob)(base64).split('').map(function (c) {
        return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
    }).join(''));
    const parsed = JSON.parse(jsonPayload);
    const _a = parsed['https://hasura.io/jwt/claims'] || {}, { 'x-hasura-allowed-roles': roles, 'x-hasura-default-role': role, 'x-hasura-user-id': userId } = _a, other = __rest(_a, ['x-hasura-allowed-roles', 'x-hasura-default-role', 'x-hasura-user-id']);
    return Object.assign({ userId: +userId, role, roles }, other);
}
exports.parseJwt = parseJwt;
;
class DeepClient {
    constructor(options) {
        var _a, _b, _c;
        this._serialize = {
            link: {
                value: 'value',
                relations: {
                    string: 'value',
                    number: 'value',
                    object: 'value',
                    to: 'link',
                    from: 'link',
                    in: 'link',
                    out: 'link',
                    type: 'link',
                    typed: 'link',
                },
            },
            value: {
                relations: {
                    link: 'link',
                },
            },
        };
        this._boolExpFields = {
            _and: true,
            _not: true,
            _or: true,
        };
        this.deep = options.deep;
        if (!this.apolloClient)
            this.apolloClient = options.apolloClient;
        if (!this.deep && !options.apolloClient)
            throw new Error('options.apolloClient or options.deep is required');
        if (this.deep && !this.apolloClient && !options.apolloClient && options.token) {
            this.apolloClient = (0, client_2.generateApolloClient)({
                path: (_a = this.deep.apolloClient) === null || _a === void 0 ? void 0 : _a.path,
                ssl: (_b = this.deep.apolloClient) === null || _b === void 0 ? void 0 : _b.ssl,
                token: options.token,
            });
        }
        if (!this.apolloClient)
            throw new Error('apolloClient is invalid');
        this.minilinks = options.minilinks || (0, minilinks_1.minilinks)([]);
        this.table = options.table || 'links';
        this.linkId = options.linkId;
        this.token = options.token;
        this.handleAuth = (options === null || options === void 0 ? void 0 : options.handleAuth) || ((_c = options === null || options === void 0 ? void 0 : options.deep) === null || _c === void 0 ? void 0 : _c.handleAuth);
        this.selectReturning = options.selectReturning || 'id type_id from_id to_id value';
        this.insertReturning = options.insertReturning || 'id';
        this.updateReturning = options.updateReturning || 'id';
        this.deleteReturning = options.deleteReturning || 'id';
        this.defaultSelectName = options.defaultSelectName || 'SELECT';
        this.defaultInsertName = options.defaultInsertName || 'INSERT';
        this.defaultUpdateName = options.defaultUpdateName || 'UPDATE';
        this.defaultDeleteName = options.defaultDeleteName || 'DELETE';
        this.silent = options.silent || false;
    }
    _silent(options = {}) {
        return typeof (options.silent) === 'boolean' ? options.silent : this.silent;
    }
    stringify(any) {
        if (typeof (any) === 'string') {
            let json;
            try {
                json = JSON.parse(any);
            }
            catch (e) { }
            return json ? JSON.stringify(json, null, 2) : any.toString();
        }
        else if (typeof (any) === 'object') {
            return JSON.stringify(any, null, 2);
        }
        return any;
    }
    select(exp, options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!exp) {
                return { error: { message: '!exp' }, data: undefined, loading: false, networkStatus: undefined };
            }
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serializeWhere(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.selectReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultSelectName;
            const q = yield this.apolloClient.query((0, gql_1.generateQuery)({
                queries: [
                    (0, gql_1.generateQueryData)({
                        tableName: table,
                        returning,
                        variables: Object.assign(Object.assign({}, variables), { where })
                    }),
                ],
                name: name,
            }));
            return Object.assign(Object.assign({}, q), { data: (_b = (_a = (q)) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.q0 });
        });
    }
    ;
    insert(objects = {}, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            const _objects = Object.prototype.toString.call(objects) === '[object Array]' ? objects : [objects];
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.insertReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultInsertName;
            let q = {};
            try {
                q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                    actions: [(0, gql_1.insertMutation)(table, Object.assign(Object.assign({}, variables), { objects: objects }), { tableName: table, operation: 'insert', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    update(exp, value, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serializeWhere(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.updateReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultUpdateName;
            let q;
            try {
                q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                    actions: [(0, gql_1.updateMutation)(table, Object.assign(Object.assign({}, variables), { where, _set: value }), { tableName: table, operation: 'update', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    delete(exp, options) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        return __awaiter(this, void 0, void 0, function* () {
            if (!exp)
                throw new Error('!exp');
            const where = typeof (exp) === 'object' ? Object.prototype.toString.call(exp) === '[object Array]' ? { id: { _in: exp } } : this.serializeWhere(exp, (options === null || options === void 0 ? void 0 : options.table) === this.table || !(options === null || options === void 0 ? void 0 : options.table) ? 'link' : 'value') : { id: { _eq: exp } };
            const table = (options === null || options === void 0 ? void 0 : options.table) || this.table;
            const returning = (options === null || options === void 0 ? void 0 : options.returning) || this.deleteReturning;
            const variables = options === null || options === void 0 ? void 0 : options.variables;
            const name = (options === null || options === void 0 ? void 0 : options.name) || this.defaultDeleteName;
            let q;
            try {
                q = yield this.apolloClient.mutate((0, gql_1.generateSerial)({
                    actions: [(0, gql_1.deleteMutation)(table, Object.assign(Object.assign({}, variables), { where, returning }), { tableName: table, operation: 'delete', returning })],
                    name: name,
                }));
            }
            catch (e) {
                const sqlError = (_d = (_c = (_b = (_a = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.extensions) === null || _c === void 0 ? void 0 : _c.internal) === null || _d === void 0 ? void 0 : _d.error;
                if (sqlError === null || sqlError === void 0 ? void 0 : sqlError.message)
                    e.message = sqlError.message;
                if (!this._silent(options))
                    throw e;
                return Object.assign(Object.assign({}, q), { data: (_g = (_f = (_e = (q)) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.m0) === null || _g === void 0 ? void 0 : _g.returning, error: e });
            }
            return Object.assign(Object.assign({}, q), { data: (_k = (_j = (_h = (q)) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.m0) === null || _k === void 0 ? void 0 : _k.returning });
        });
    }
    ;
    reserve(count) {
        return (0, reserve_1.reserve)({ count, client: this.apolloClient });
    }
    ;
    await(id, options = { results: false }) {
        return __awaiter(this, void 0, void 0, function* () {
            return (0, promise_1.awaitPromise)({
                id, client: this.apolloClient,
                Then: yield this.id('@deep-foundation/core', 'Then'),
                Promise: yield this.id('@deep-foundation/core', 'Promise'),
                Resolved: yield this.id('@deep-foundation/core', 'Resolved'),
                Rejected: yield this.id('@deep-foundation/core', 'Rejected'),
                Results: options.results
            });
        });
    }
    ;
    serializeWhere(exp, env = 'link') {
        var _a, _b, _c, _d, _e, _f;
        if (Object.prototype.toString.call(exp) === '[object Array]')
            return exp.map((e) => this.serializeWhere(e, env));
        else if (typeof (exp) === 'object') {
            const keys = Object.keys(exp);
            const result = {};
            for (let k = 0; k < keys.length; k++) {
                const key = keys[k];
                const type = typeof (exp[key]);
                let setted = false;
                if (env === 'link') {
                    if (type === 'string' || type === 'number') {
                        if (key === 'value' || key === type) {
                            setted = result[type] = { value: { _eq: exp[key] } };
                        }
                        else {
                            setted = result[key] = { _eq: exp[key] };
                        }
                    }
                    else if (!this._boolExpFields[key] && Object.prototype.toString.call(exp[key]) === '[object Array]') {
                        setted = result[key] = this.serializeWhere(this.pathToWhere(...exp[key]));
                    }
                    else if (type === 'object' && exp[key].hasOwnProperty('_type_of') && !!~['type_id', 'from_id', 'to_id'].indexOf(key)) {
                        setted = result[key.slice(0, -3)] = { _by_item: { path_item_id: { _eq: exp[key]._type_of }, group_id: { _eq: 0 } } };
                    }
                }
                else if (env === 'value') {
                    if (type === 'string' || type === 'number') {
                        setted = result[key] = { _eq: exp[key] };
                    }
                }
                if (!setted)
                    result[key] = this._boolExpFields[key] ? this.serializeWhere(exp[key], env) : ((_c = (_b = (_a = this._serialize) === null || _a === void 0 ? void 0 : _a[env]) === null || _b === void 0 ? void 0 : _b.relations) === null || _c === void 0 ? void 0 : _c[key]) ? this.serializeWhere(exp[key], (_f = (_e = (_d = this._serialize) === null || _d === void 0 ? void 0 : _d[env]) === null || _e === void 0 ? void 0 : _e.relations) === null || _f === void 0 ? void 0 : _f[key]) : exp[key];
            }
            return result;
        }
        else {
            if (typeof (exp) === 'undefined')
                throw new Error('undefined in query');
            return exp;
        }
    }
    ;
    serializeQuery(exp, env = 'link') {
        const { limit, order_by, offset, distinct_on } = exp, where = __rest(exp, ["limit", "order_by", "offset", "distinct_on"]);
        return { limit, order_by, offset, distinct_on, where: this.serializeWhere(where, env) };
    }
    pathToWhere(start, ...path) {
        const pckg = typeof (start) === 'string' ? { type_id: exports.GLOBAL_ID_PACKAGE, value: start } : { id: start };
        let where = pckg;
        for (let p = 0; p < path.length; p++) {
            const item = path[p];
            if (typeof (item) !== 'boolean') {
                const nextWhere = { in: { type_id: exports.GLOBAL_ID_CONTAIN, value: item, from: where } };
                where = nextWhere;
            }
        }
        return where;
    }
    id(start, ...path) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const q = yield this.select(this.pathToWhere(start, ...path));
            if (q.error) {
                throw q.error;
            }
            const result = (((_b = (_a = q === null || q === void 0 ? void 0 : q.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id) | ((_c = _ids === null || _ids === void 0 ? void 0 : _ids[start]) === null || _c === void 0 ? void 0 : _c[path === null || path === void 0 ? void 0 : path[0]]) | 0);
            if (!result && path[path.length - 1] !== true) {
                throw new Error(`Id not found by [${JSON.stringify([start, ...path])}]`);
            }
            return result;
        });
    }
    ;
    guest(options = {}) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const relogin = typeof (options.relogin) === 'boolean' ? options.relogin : true;
            const result = yield this.apolloClient.query({ query: exports.GUEST });
            const { linkId, token, error } = ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.guest) || {};
            if (!error && !!token && relogin) {
                if (this === null || this === void 0 ? void 0 : this.handleAuth)
                    setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
            }
            return { linkId, token, error: !error && (!linkId || !token) ? 'unexepted' : error };
        });
    }
    ;
    jwt(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const relogin = typeof (options.relogin) === 'boolean' ? options.relogin : false;
            if (options === null || options === void 0 ? void 0 : options.token) {
                try {
                    const token = options === null || options === void 0 ? void 0 : options.token;
                    const decoded = parseJwt(token);
                    const linkId = decoded === null || decoded === void 0 ? void 0 : decoded.userId;
                    if (!!token && relogin) {
                        if (this === null || this === void 0 ? void 0 : this.handleAuth)
                            setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
                    }
                    return { linkId, token, error: (!linkId || !token) ? 'unexepted' : undefined };
                }
                catch (e) {
                    return { error: e };
                }
            }
            else if (options === null || options === void 0 ? void 0 : options.linkId) {
                const result = yield this.apolloClient.query({ query: exports.JWT, variables: { linkId: +options.linkId } });
                const { linkId, token, error } = ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.jwt) || {};
                if (!error && !!token && relogin) {
                    if (this === null || this === void 0 ? void 0 : this.handleAuth)
                        setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(+linkId, token), 0);
                }
                return { linkId, token, error: error ? error : (!linkId) ? 'unexepted' : undefined };
            }
            else
                return { error: `linkId or token must be provided` };
        });
    }
    ;
    whoami() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.apolloClient.query({ query: exports.WHOISME });
            this.linkId = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.jwt) === null || _b === void 0 ? void 0 : _b.linkId;
            return (_d = (_c = result === null || result === void 0 ? void 0 : result.data) === null || _c === void 0 ? void 0 : _c.jwt) === null || _d === void 0 ? void 0 : _d.linkId;
        });
    }
    login(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.jwt(Object.assign(Object.assign({}, options), { relogin: true }));
        });
    }
    ;
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this === null || this === void 0 ? void 0 : this.handleAuth)
                setTimeout(() => this === null || this === void 0 ? void 0 : this.handleAuth(0, ''), 0);
            return { linkId: 0, token: '' };
        });
    }
    ;
    can(objectIds, subjectIds, actionIds, userIds = this.linkId) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const where = {
                object_id: typeof (objectIds) === 'number' ? { _eq: +objectIds } : { _in: objectIds },
                subject_id: typeof (subjectIds) === 'number' ? { _eq: +subjectIds } : { _in: subjectIds },
                action_id: typeof (actionIds) === 'number' ? { _eq: +actionIds } : { _in: actionIds },
            };
            const result = yield this.select(where, { table: 'can', returning: 'rule_id' });
            return !!((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.length);
        });
    }
}
exports.DeepClient = DeepClient;
exports.JWT = (0, client_1.gql) `query JWT($linkId: Int) {
  jwt(input: {linkId: $linkId}) {
    linkId
    token
  }
}`;
exports.WHOISME = (0, client_1.gql) `query WHOISME {
  jwt(input: {}) {
    linkId
  }
}`;
exports.GUEST = (0, client_1.gql) `query GUEST {
  guest {
    linkId
    token
  }
}`;
function useAuthNode() {
    return (0, local_1.useLocalStore)('use_auth_link_id', 0);
}
exports.useAuthNode = useAuthNode;
function useDeep(apolloClientProps) {
    const apolloClientHook = (0, client_1.useApolloClient)();
    const apolloClient = apolloClientProps || apolloClientHook;
    const [linkId, setLinkId] = useAuthNode();
    const [token, setToken] = (0, react_token_1.useTokenController)();
    const deep = (0, react_1.useMemo)(() => {
        if (!(apolloClient === null || apolloClient === void 0 ? void 0 : apolloClient.jwt_token)) {
            log({ token, apolloClient });
        }
        console.log({ linkId, token });
        return new DeepClient({
            apolloClient, linkId, token,
            handleAuth: (linkId, token) => {
                setToken(token);
                setLinkId(linkId);
            },
        });
    }, [apolloClient]);
    return deep;
}
exports.useDeep = useDeep;
function useDeepQuery(query, options) {
    const deep = useDeep();
    const wq = (0, react_1.useMemo)(() => {
        const sq = deep.serializeQuery(query);
        return (0, gql_1.generateQuery)({
            operation: 'query',
            queries: [(0, gql_1.generateQueryData)({
                    tableName: 'links',
                    returning: `
          id type_id from_id to_id value
          string { id value }
          number { id value }
          object { id value }
        `,
                    variables: Object.assign(Object.assign({}, sq), options === null || options === void 0 ? void 0 : options.variables)
                })],
            name: (options === null || options === void 0 ? void 0 : options.name) || 'USE_DEEP_QUERY',
        });
    }, []);
    return (0, client_1.useQuery)(wq.query, { variables: wq === null || wq === void 0 ? void 0 : wq.variables });
}
exports.useDeepQuery = useDeepQuery;
//# sourceMappingURL=client.js.map