"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerController = exports.runnerControllerOptionsDefault = void 0;
const child_process_1 = require("child_process");
const crypto_1 = __importDefault(require("crypto"));
const axios_1 = __importDefault(require("axios"));
const get_port_1 = __importDefault(require("get-port"));
const debug_1 = __importDefault(require("debug"));
const util_1 = __importDefault(require("util"));
const execAsync = util_1.default.promisify(child_process_1.exec);
const debug = (0, debug_1.default)('deeplinks:container-controller');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const DOCKER = process.env.DOCKER || '0';
exports.runnerControllerOptionsDefault = {
    gql_docker_domain: 'links',
    gql_port_path: '3006/gql',
    network: 'network',
    handlersHash: {},
};
const delay = ms => new Promise(resolve => setTimeout(resolve, ms));
class ContainerController {
    constructor(options) {
        this.runContainerHash = {};
        this.handlersHash = {};
        this.network = (options === null || options === void 0 ? void 0 : options.network) || exports.runnerControllerOptionsDefault.network;
        this.gql_docker_domain = (options === null || options === void 0 ? void 0 : options.gql_docker_domain) || exports.runnerControllerOptionsDefault.gql_docker_domain;
        this.gql_port_path = (options === null || options === void 0 ? void 0 : options.gql_port_path) || exports.runnerControllerOptionsDefault.gql_port_path;
        this.handlersHash = (options === null || options === void 0 ? void 0 : options.handlersHash) || exports.runnerControllerOptionsDefault.handlersHash;
    }
    ;
    _runContainer(containerName, dockerPort, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handler, forcePort, forceRestart, publish } = options;
            const { network, gql_docker_domain, gql_port_path } = this;
            let done = false;
            let count = 30;
            let dockerRunResult;
            while (!done) {
                log('_runContainer count', { count });
                if (count < 0)
                    return { error: 'timeout _runContainer' };
                count--;
                try {
                    const command = `docker pull ${handler}; docker run -e PORT=${dockerPort} -e GQL_URN=deep${yield this.getDelimiter()}${gql_docker_domain}${yield this.getDelimiter()}1:${gql_port_path} -e GQL_SSL=0 --name ${containerName} ${publish ? `-p ${dockerPort}:${dockerPort}` : `--expose ${dockerPort}`} --net ${network} -d ${handler}`;
                    log('_runContainer command', { command });
                    const dockerRunResultObject = yield execAsync(command);
                    log('_runContainer dockerRunResultObject', JSON.stringify(dockerRunResultObject, null, 2));
                    dockerRunResult = dockerRunResultObject === null || dockerRunResultObject === void 0 ? void 0 : dockerRunResultObject.stdout;
                    log('_runContainer dockerRunResultString', { dockerRunResult });
                }
                catch (e) {
                    error('_runContainer error', e);
                    dockerRunResult = e.stderr;
                }
                if (dockerRunResult.indexOf('port is already allocated') !== -1) {
                    if (forcePort)
                        return { error: 'port is already allocated' };
                    continue;
                }
                else if (dockerRunResult.indexOf('is already in use by container') !== -1) {
                    if (!forceRestart)
                        return { error: 'is already in use by container' };
                    yield this._dropContainer(containerName);
                }
                else {
                    done = true;
                    if (!publish && !+DOCKER)
                        throw new Error('Need proxy, dl issue https://github.com/deep-foundation/deeplinks/issues/45');
                    let host = publish ? +DOCKER ? 'host.docker.internal' : 'localhost' : +DOCKER ? containerName : '';
                    const container = { name: containerName, host, port: dockerPort };
                    log('_runContainer container', container);
                    const waitResult = yield execAsync(`npx wait-on --timeout 5000 http-get://${host}:${dockerPort}/healthz`);
                    log('_runContainer npx done', { waitResult });
                    return container;
                }
            }
        });
    }
    _waitContainer(containerName) {
        return __awaiter(this, void 0, void 0, function* () {
            const { runContainerHash } = this;
            let done = false;
            let count = 100;
            while (!done) {
                log('_waitContainer count', { count });
                if (count < 0)
                    return { error: 'timeout _waitContainer' };
                count--;
                if (runContainerHash[containerName]) {
                    yield delay(1000);
                }
                else {
                    done = true;
                }
            }
        });
    }
    getDelimiter() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this.delimiter)
                return this.delimiter;
            const versionResult = yield execAsync(`docker-compose version --short`);
            log('getDelimiter versionResult', { versionResult });
            const majorVersion = (_a = versionResult === null || versionResult === void 0 ? void 0 : versionResult.stdout) === null || _a === void 0 ? void 0 : _a.match(/\d+/)[0];
            log('getDelimiter majorVersion', { majorVersion });
            this.delimiter = majorVersion === '1' ? '_' : '-';
            return this.delimiter;
        });
    }
    newContainer(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handler, forcePort, forceName } = options;
            const { network, handlersHash, runContainerHash } = this;
            log('newContainer options, network, handlersHash', { options, network, handlersHash });
            const containerName = forceName || `deep${yield this.getDelimiter()}${crypto_1.default.createHash('md5').update(handler).digest("hex")}`;
            log('newContainer containerName, forceName', { containerName, forceName });
            let container = yield this.findContainer(containerName);
            if (container)
                return container;
            let dockerPort = forcePort || (yield (0, get_port_1.default)());
            let done = false;
            let count = 30;
            while (!done) {
                log('newContainer count newContainer', { count });
                if (count < 0)
                    return { error: 'timeout findPort' };
                count--;
                if (runContainerHash[containerName]) {
                    yield this._waitContainer(containerName);
                    if (!!handlersHash[containerName]) {
                        container = handlersHash[containerName];
                    }
                }
                else {
                    runContainerHash[containerName] = true;
                    container = yield this._runContainer(containerName, dockerPort, options);
                    log('newContainer _runContainer result', { container });
                }
                if (!!container && !(container === null || container === void 0 ? void 0 : container.error)) {
                    done = true;
                    handlersHash[containerName] = container;
                    runContainerHash[containerName] = undefined;
                    log('newContainer done true', { containerName, handlersHash, runContainerHash });
                }
                if (!done && !forcePort)
                    dockerPort = yield (0, get_port_1.default)();
            }
            log('newContainer hashes', { handlersHash, runContainerHash });
            return container;
        });
    }
    findContainer(containerName) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handlersHash, runContainerHash } = this;
            log('findContainer hashes', { handlersHash, runContainerHash });
            if (runContainerHash[containerName])
                yield this._waitContainer(containerName);
            log('findContainer result', handlersHash[containerName]);
            return handlersHash[containerName];
        });
    }
    _dropContainer(containerName) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            log('_dropContainer', { containerName });
            return (_a = (yield execAsync(`docker stop ${containerName} && docker rm ${containerName}`))) === null || _a === void 0 ? void 0 : _a.stdout;
        });
    }
    dropContainer(container) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handlersHash, runContainerHash } = this;
            if (runContainerHash[container.name])
                yield this._waitContainer(container.name);
            if (!handlersHash[container.name])
                return;
            let dockerStopResult = yield this._dropContainer(container.name);
            handlersHash[container.name] = undefined;
            log('dropContainer dockerStopResult', { dockerStopResult });
        });
    }
    initHandler(container) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const { host, port } = container;
            let initResult;
            try {
                const initRunner = `http://${host}:${port}/init`;
                log('initHandler initRunner', { initRunner });
                initResult = yield axios_1.default.post(initRunner);
                log('initHandler initResult', { initResult });
            }
            catch (e) {
                error('initHandler error', e);
                return ({ error: e });
            }
            if ((_a = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _a === void 0 ? void 0 : _a.error)
                return { error: (_b = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _b === void 0 ? void 0 : _b.error };
            return {};
        });
    }
    callHandler(options) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const { container } = options;
            try {
                const callRunner = `http://${container.host}:${container.port}/call`;
                log('callHandler', { callRunner, params: options });
                const callResult = yield axios_1.default.post(callRunner, { params: options });
                log('callHandler callResult', { callResult });
                if ((_a = callResult === null || callResult === void 0 ? void 0 : callResult.data) === null || _a === void 0 ? void 0 : _a.error)
                    return { error: (_b = callResult === null || callResult === void 0 ? void 0 : callResult.data) === null || _b === void 0 ? void 0 : _b.error };
                if ((_c = callResult === null || callResult === void 0 ? void 0 : callResult.data) === null || _c === void 0 ? void 0 : _c.resolved) {
                    return callResult.data.resolved;
                }
                return Promise.reject((_d = callResult === null || callResult === void 0 ? void 0 : callResult.data) === null || _d === void 0 ? void 0 : _d.rejected);
            }
            catch (e) {
                error('error', e);
                return ({ error: e });
            }
        });
    }
}
exports.ContainerController = ContainerController;
//# sourceMappingURL=container-controller.js.map