"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handlePort = exports.handleSchedule = exports.handleSelectorOperation = exports.handleOperation = exports.handlerOperations = exports.useRunner = exports.getJwt = exports.containerController = exports.processPromises = exports.makePromiseResult = exports.api = exports.DOCKER = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const apollo_boost_1 = require("apollo-boost");
const promise_1 = require("../promise");
const client_2 = require("../client");
const global_ids_1 = require("../global-ids");
const container_controller_1 = require("../container-controller");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deeplinks:eh:links');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const DOCKER_DEEPLINKS_URL = process.env.DOCKER_DEEPLINKS_URL || 'http://host.docker.internal:3006';
exports.DOCKER = process.env.DOCKER || '0';
exports.api = new api_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
function makePromiseResult(promiseId, resolvedTypeId, promiseResultTypeId, result, promiseReasonTypeId, handleInsertId) {
    if (typeof handleInsertId === 'number') {
        return {
            from: {
                data: {
                    from_id: promiseId,
                    type_id: resolvedTypeId,
                    to: {
                        data: {
                            type_id: promiseResultTypeId,
                            object: { data: { value: result } },
                        }
                    }
                }
            },
            type_id: promiseReasonTypeId,
            to_id: handleInsertId
        };
    }
    else {
        return {
            from_id: promiseId,
            type_id: resolvedTypeId,
            to: {
                data: {
                    type_id: promiseResultTypeId,
                    object: { data: { value: result } },
                }
            }
        };
    }
}
exports.makePromiseResult = makePromiseResult;
;
function processPromises(promises, handleInsertsIds, promiseId, promiseResultTypeId, promiseReasonTypeId, resolvedTypeId, rejectedTypeId, log) {
    return __awaiter(this, void 0, void 0, function* () {
        log("promises.length: ", promises.length);
        yield Promise.allSettled(promises.map((p) => p()))
            .then((values) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            log("values: ", values);
            const promiseResults = [];
            for (let i = 0; i < values.length; i++) {
                const value = values[i];
                const handleInsertId = handleInsertsIds[i];
                let resultTypeId = null;
                let result = null;
                if (value.status == 'fulfilled') {
                    result = value.value;
                    resultTypeId = resolvedTypeId;
                }
                if (value.status == 'rejected') {
                    result = value.reason;
                    resultTypeId = rejectedTypeId;
                }
                log("result: ", result);
                log("resultTypeId: ", resultTypeId);
                promiseResults.push(makePromiseResult(promiseId, resultTypeId, promiseResultTypeId, result, promiseReasonTypeId, handleInsertId));
            }
            try {
                yield deep.insert(promiseResults, { name: 'IMPORT_PROMISES_RESULTS' });
                log("inserted promiseResults: ", JSON.stringify(promiseResults, null, 2));
            }
            catch (e) {
                log('promiseResults insert error: ', (_a = e === null || e === void 0 ? void 0 : e.message) !== null && _a !== void 0 ? _a : e);
            }
        }));
    });
}
exports.processPromises = processPromises;
exports.containerController = new container_controller_1.ContainerController({
    gql_docker_domain: +exports.DOCKER ? 'links' : 'graphql-engine',
    gql_port_path: +exports.DOCKER ? '3006/gql' : '8080/v1/graphql',
    network: 'deep_network',
    handlersHash: {}
});
function getJwt(handlerId, useRunnerDebug) {
    return __awaiter(this, void 0, void 0, function* () {
        const userTypeId = yield deep.id('@deep-foundation/core', 'User');
        useRunnerDebug("userTypeId: ", JSON.stringify(userTypeId, null, 2));
        const packageTypeId = yield deep.id('@deep-foundation/core', 'Package');
        useRunnerDebug("packageTypeId: ", JSON.stringify(packageTypeId, null, 2));
        const queryString = `query {
    mpUp: mp(where: {
      item_id: {_eq: "${handlerId}"},
      path_item: {type_id: {_in: ["${userTypeId}", "${packageTypeId}"]}}
    }) {
      id
      path_item {
        id
        type_id
        value
      }
      path_item_depth
      position_id
    }
    mpMe: mp(where: {item_id: {_eq: "${handlerId}"}, path_item_id: { _eq: "${handlerId}" }}) {
      id
      path_item_depth
      position_id
    }
  }`;
        const ownerResults = yield client.query({ query: (0, apollo_boost_1.gql) `${queryString}` });
        useRunnerDebug("ownerResults: ", JSON.stringify(ownerResults, null, 2));
        const mpUp = ownerResults.data.mpUp;
        const mpMe = ownerResults.data.mpMe;
        const possibleOwners = mpMe.map((me) => {
            const getDepthDifference = (depth) => me.path_item_depth - depth;
            const up = mpUp.filter((up) => up.position_id == me.position_id);
            const closestUp = up.sort((a, b) => getDepthDifference(a.path_item_depth) - getDepthDifference(b.path_item_depth))[0];
            return closestUp === null || closestUp === void 0 ? void 0 : closestUp.path_item;
        }).filter(r => !!r);
        useRunnerDebug("possibleOwners: ", JSON.stringify(possibleOwners, null, 2));
        const ownerPackage = possibleOwners.find(r => r.type_id == packageTypeId);
        const ownerUser = possibleOwners.find(r => r.type_id == userTypeId);
        let ownerId;
        if (ownerPackage) {
            ownerId = ownerPackage.id;
            useRunnerDebug("owner is package");
        }
        else if (ownerUser) {
            ownerId = ownerUser.id;
            useRunnerDebug("owner is user");
        }
        else {
            throw new Error("No handler owner found.");
        }
        useRunnerDebug("ownerId: ", ownerId);
        const jwt = (yield deep.jwt({ linkId: ownerId })).token;
        useRunnerDebug('jwt', jwt);
        return jwt;
    });
}
exports.getJwt = getJwt;
const useRunner = ({ code, handler, handlerId, oldLink, newLink, moment, promiseId, }) => __awaiter(void 0, void 0, void 0, function* () {
    const useRunnerDebug = (0, debug_1.default)('deeplinks:eh:links:useRunner');
    useRunnerDebug("handler4: ");
    const jwt = yield getJwt(handlerId, useRunnerDebug);
    const container = yield exports.containerController.newContainer({ publish: +exports.DOCKER ? false : true, forceRestart: true, handler, code, jwt, data: { oldLink, newLink, moment } });
    useRunnerDebug('newContainerResult', container);
    const initResult = yield exports.containerController.initHandler(container);
    useRunnerDebug('initResult', initResult);
    const callResult = yield exports.containerController.callHandler({ code, container, jwt, data: { oldLink, newLink, moment, promiseId } });
    useRunnerDebug('callResult', callResult);
    return callResult;
});
exports.useRunner = useRunner;
exports.handlerOperations = {
    Insert: 'HandleInsert',
    Update: 'HandleUpdate',
    Delete: 'HandleDelete',
};
function handleOperation(operation, oldLink, newLink) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    return __awaiter(this, void 0, void 0, function* () {
        const handleOperationDebug = (0, debug_1.default)('deeplinks:eh:links:handleOperation');
        const current = newLink !== null && newLink !== void 0 ? newLink : oldLink;
        const currentLinkId = current.id;
        const currentTypeId = current.type_id;
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promise = yield (0, promise_1.findPromiseLink)({
            id: currentLinkId, client: deep.apolloClient,
            Then: yield deep.id('@deep-foundation/core', 'Then'),
            Promise: yield deep.id('@deep-foundation/core', 'Promise'),
            Resolved: resolvedTypeId,
            Rejected: rejectedTypeId,
            Results: false,
        });
        if (promise) {
            const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
            const handleOperationTypeId = yield deep.id('@deep-foundation/core', exports.handlerOperations[operation]);
            const dockerSupportsJsType = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
            const queryString = `query SELECT_CODE($typeId: bigint) { links(where: {
            type_id: { _eq: ${yield deep.id('@deep-foundation/core', 'SyncTextFile')} },
            in: {
              from_id: { _eq: ${dockerSupportsJsType} },
              type_id: { _eq: ${handlerTypeId} },
              in: {
                from_id: { _eq: $typeId },
                type_id: { _eq: ${handleOperationTypeId} },
              }
            }
          }) {
            id
            value
            in(where: { type_id: { _eq: ${handlerTypeId} } }) {
              id
              in(where: { type_id: { _eq: ${handleOperationTypeId} } }) {
                id
              }
              support: from {
                id
                isolation: from {
                  id
                  value
                }
              }
            }
          } }`;
            const query = (0, apollo_boost_1.gql) `${queryString}`;
            const variables = {
                typeId: currentTypeId
            };
            const handlersResult = yield client.query({ query, variables });
            const promises = [];
            const handleInsertsIds = [];
            const handlersWithCode = (_a = handlersResult === null || handlersResult === void 0 ? void 0 : handlersResult.data) === null || _a === void 0 ? void 0 : _a.links;
            const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
            const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
            if ((handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length) > 0) {
                handleOperationDebug("handlersWithCode: ", JSON.stringify(handlersWithCode, null, 2));
                handleOperationDebug("handlersWithCode?.length: ", handlersWithCode === null || handlersWithCode === void 0 ? void 0 : handlersWithCode.length);
                for (const handlerWithCode of handlersWithCode) {
                    const code = (_b = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.value) === null || _b === void 0 ? void 0 : _b.value;
                    const isolationValue = (_g = (_f = (_e = (_d = (_c = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.support) === null || _e === void 0 ? void 0 : _e.isolation) === null || _f === void 0 ? void 0 : _f.value) === null || _g === void 0 ? void 0 : _g.value;
                    const handlerId = (_j = (_h = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.id;
                    const handleInsertId = (_m = (_l = (_k = handlerWithCode === null || handlerWithCode === void 0 ? void 0 : handlerWithCode.in) === null || _k === void 0 ? void 0 : _k[0]) === null || _l === void 0 ? void 0 : _l.in) === null || _m === void 0 ? void 0 : _m[0].id;
                    if (code && isolationValue && handlerId && handleInsertId) {
                        try {
                            promises.push(() => __awaiter(this, void 0, void 0, function* () { return (0, exports.useRunner)({ code, handlerId, handler: isolationValue, oldLink, newLink, promiseId: promise.id }); }));
                            handleInsertsIds.push(handleInsertId);
                        }
                        catch (error) {
                            handleOperationDebug('error', error);
                        }
                    }
                    else {
                        promises.push(() => __awaiter(this, void 0, void 0, function* () { return Promise.reject(new Error('Code of a handler is not loaded.')); }));
                        handleInsertsIds.push(null);
                    }
                }
                yield processPromises(promises, handleInsertsIds, promise.id, promiseResultTypeId, promiseReasonTypeId, resolvedTypeId, rejectedTypeId, handleOperationDebug);
            }
        }
    });
}
exports.handleOperation = handleOperation;
function handleSelectorOperation(operation, oldLink, newLink) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o;
    return __awaiter(this, void 0, void 0, function* () {
        const handleSelectorDebug = debug.extend('handleSelector').extend('log');
        const current = newLink !== null && newLink !== void 0 ? newLink : oldLink;
        const currentLinkId = current.id;
        const handleOperationTypeId = yield deep.id('@deep-foundation/core', exports.handlerOperations[operation]);
        handleSelectorDebug('handleOperation', operation);
        const promiseSelectorsQueryString = `query SELECT_PROMISE_SELECTORS($itemId: bigint) { promise_selectors(where: {
    item_id: { _eq: $itemId },
    handle_operation: { type_id: { _eq: ${handleOperationTypeId} } }
  }) {
    id
    promise_id
    handle_operation {
      id
      handler: to {
        id
        supports: from {
          id
          isolation: from {
            id
            image: value
          }
        }
        file: to {
          id
          code: value
        }
      }
    }
  } }`;
        const promiseSelectorsQuery = (0, apollo_boost_1.gql) `${promiseSelectorsQueryString}`;
        const promiseSelectorsQueryVariables = {
            itemId: currentLinkId
        };
        const promiseSelectorsResult = yield client.query({ query: promiseSelectorsQuery, variables: promiseSelectorsQueryVariables });
        const promiseSelectors = (_a = promiseSelectorsResult === null || promiseSelectorsResult === void 0 ? void 0 : promiseSelectorsResult.data) === null || _a === void 0 ? void 0 : _a.promise_selectors;
        handleSelectorDebug('promiseSelectors.length', promiseSelectors === null || promiseSelectors === void 0 ? void 0 : promiseSelectors.length);
        if (!(promiseSelectors === null || promiseSelectors === void 0 ? void 0 : promiseSelectors.length)) {
            return;
        }
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResultTypeId = yield deep.id('@deep-foundation/core', 'PromiseResult');
        const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
        const promiseSelectorsByPromiseId = promiseSelectors.reduce((accumulator, current) => {
            const promiseId = current.promise_id;
            if (!accumulator[promiseId]) {
                accumulator[promiseId] = [];
            }
            accumulator[promiseId].push(current);
            return accumulator;
        }, {});
        handleSelectorDebug('promiseSelectorsByPromiseId', JSON.stringify(promiseSelectorsByPromiseId, null, 2));
        for (const promiseIdString in promiseSelectorsByPromiseId) {
            const promiseSelectors = promiseSelectorsByPromiseId[promiseIdString];
            const promiseSelectorsIds = promiseSelectors.map(promiseSelector => promiseSelector.id);
            handleSelectorDebug('promiseSelectorsIds', JSON.stringify(promiseSelectorsIds, null, 2));
            const promiseId = parseInt(promiseIdString);
            const promises = [];
            const handleInsertsIds = [];
            for (const promiseSelector of promiseSelectors) {
                const code = (_e = (_d = (_c = (_b = promiseSelector === null || promiseSelector === void 0 ? void 0 : promiseSelector.handle_operation) === null || _b === void 0 ? void 0 : _b.handler) === null || _c === void 0 ? void 0 : _c.file) === null || _d === void 0 ? void 0 : _d.code) === null || _e === void 0 ? void 0 : _e.value;
                const isolationValue = (_k = (_j = (_h = (_g = (_f = promiseSelector === null || promiseSelector === void 0 ? void 0 : promiseSelector.handle_operation) === null || _f === void 0 ? void 0 : _f.handler) === null || _g === void 0 ? void 0 : _g.supports) === null || _h === void 0 ? void 0 : _h.isolation) === null || _j === void 0 ? void 0 : _j.image) === null || _k === void 0 ? void 0 : _k.value;
                const handlerId = (_m = (_l = promiseSelector === null || promiseSelector === void 0 ? void 0 : promiseSelector.handle_operation) === null || _l === void 0 ? void 0 : _l.handler) === null || _m === void 0 ? void 0 : _m.id;
                const handleInsertId = (_o = promiseSelector === null || promiseSelector === void 0 ? void 0 : promiseSelector.handle_operation) === null || _o === void 0 ? void 0 : _o.id;
                if (code && isolationValue && handlerId && handleInsertId) {
                    try {
                        promises.push(() => __awaiter(this, void 0, void 0, function* () { return (0, exports.useRunner)({ code, handlerId, handler: isolationValue, oldLink, newLink, promiseId }); }));
                        handleInsertsIds.push(handleInsertId);
                    }
                    catch (error) {
                        handleSelectorDebug('error', error);
                    }
                }
                else {
                    promises.push(() => __awaiter(this, void 0, void 0, function* () { return Promise.reject(new Error('Code of a handler is not loaded.')); }));
                    handleInsertsIds.push(null);
                }
            }
            processPromises(promises, handleInsertsIds, promiseId, promiseResultTypeId, promiseReasonTypeId, resolvedTypeId, rejectedTypeId, handleSelectorDebug);
            yield deep.delete(promiseSelectorsIds, { name: 'DELETE_PROMISES_SELECTORS', table: 'promise_selectors' });
        }
    });
}
exports.handleSelectorOperation = handleSelectorOperation;
function handleSchedule(handleScheduleLink, operation) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        const handleScheduleDebug = (0, debug_1.default)('deeplinks:eh:links:handleSchedule');
        handleScheduleDebug('handleScheduleLink', handleScheduleLink);
        handleScheduleDebug('operation', operation);
        if (operation == 'INSERT') {
            const schedule = yield deep.select({
                type_id: yield deep.id('@deep-foundation/core', 'Schedule'),
                out: {
                    id: { _eq: handleScheduleLink.id },
                },
            }, {
                table: 'links',
                returning: 'id value',
            });
            handleScheduleDebug(schedule);
            const scheduleId = (_b = (_a = schedule === null || schedule === void 0 ? void 0 : schedule.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
            const scheduleValue = (_d = (_c = schedule === null || schedule === void 0 ? void 0 : schedule.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value.value;
            handleScheduleDebug('scheduleId', scheduleId);
            handleScheduleDebug('scheduleValue', scheduleValue);
            yield exports.api.query({
                type: 'create_cron_trigger',
                args: {
                    name: `handle_schedule_${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id}`,
                    webhook: `${DOCKER_DEEPLINKS_URL}/api/scheduler`,
                    schedule: scheduleValue,
                    include_in_metadata: true,
                    payload: {
                        scheduleId,
                        schedule: scheduleValue,
                        handleScheduleLinkId: handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id,
                    },
                    retry_conf: {
                        num_retries: 3,
                        timeout_seconds: 120,
                        tolerance_seconds: 21675,
                        retry_interval_seconds: 12
                    },
                    comment: `Event trigger for handle schedule link ${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id} with cron schedule definition ${scheduleValue} of ${scheduleId} schedule.`,
                }
            });
            handleScheduleDebug('cron trigger created');
        }
        else if (operation == 'DELETE') {
            yield exports.api.query({
                type: 'delete_cron_trigger',
                args: {
                    name: `handle_schedule_${handleScheduleLink === null || handleScheduleLink === void 0 ? void 0 : handleScheduleLink.id}`,
                }
            });
            handleScheduleDebug('cron trigger deleted');
        }
    });
}
exports.handleSchedule = handleSchedule;
function handlePort(handlePortLink, operation) {
    var _a, _b, _c, _d, _e, _f;
    return __awaiter(this, void 0, void 0, function* () {
        const handlePortDebug = (0, debug_1.default)('deeplinks:eh:links:handlePort');
        handlePortDebug('handlePortLink', handlePortLink);
        handlePortDebug('operation', operation);
        const port = yield deep.select({
            id: { _eq: handlePortLink.from_id },
        }, {
            table: 'links',
            returning: 'id value',
        });
        handlePortDebug(port);
        const portId = (_b = (_a = port === null || port === void 0 ? void 0 : port.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
        const portValue = (_d = (_c = port === null || port === void 0 ? void 0 : port.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value.value;
        handlePortDebug('portId', portId);
        handlePortDebug('portValue', portValue);
        if (operation == 'INSERT') {
            const isolationProvider = yield deep.select({
                type_id: yield deep.id('@deep-foundation/core', 'DockerIsolationProvider'),
                in: {
                    id: { _eq: handlePortLink.id },
                },
            }, {
                table: 'links',
                returning: 'id value',
            });
            handlePortDebug('INSERT', isolationProvider);
            const dockerImage = (_f = (_e = isolationProvider === null || isolationProvider === void 0 ? void 0 : isolationProvider.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.value.value;
            handlePortDebug('INSERT dockerImage', dockerImage);
            const containerName = `deep${yield exports.containerController.getDelimiter()}handle_port_${portValue}`;
            handlePortDebug('INSERT containerName', containerName);
            const container = yield exports.containerController.newContainer({ publish: true, forcePort: portValue, forceName: containerName, handler: dockerImage, code: null, jwt: null, data: {} });
            handlePortDebug('INSERT newContainer result', container);
            if (container.error)
                return handlePortDebug('portResult.error', container.error);
            handlePortDebug(`INSERT port handler container ${JSON.stringify(container)} created`);
        }
        else if (operation == 'DELETE') {
            const containerName = `deep${yield exports.containerController.getDelimiter()}handle_port_${portValue}`;
            handlePortDebug('DELETE containerName', containerName);
            const container = yield exports.containerController.findContainer(containerName);
            handlePortDebug('DELETE container', container);
            yield exports.containerController.dropContainer(container);
            handlePortDebug('DELETE port handler container deleted');
        }
    });
}
exports.handlePort = handlePort;
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    try {
        let handlePortId;
        try {
            handlePortId = yield deep.id('@deep-foundation/core', 'HandlePort');
        }
        catch (_k) {
            return res.status(500).json({ error: '@deep-foundation/core package is not ready to support links handlers.' });
        }
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            const oldRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            if (oldRow) {
                const queryResult = yield deep.select({
                    id: { _eq: oldRow.id },
                }, {
                    returning: `value`,
                });
                oldRow.value = (_d = (_c = queryResult.data) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value;
            }
            const newRow = (_e = event === null || event === void 0 ? void 0 : event.data) === null || _e === void 0 ? void 0 : _e.new;
            if (newRow) {
                const queryResult = yield deep.select({
                    id: { _eq: newRow.id },
                }, {
                    returning: `value`,
                });
                newRow.value = (_g = (_f = queryResult.data) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.value;
            }
            const current = operation === 'DELETE' ? oldRow : newRow;
            log(`Processing ${current.id} link.`);
            log('operation', operation);
            log('current', current);
            try {
                if (operation === 'INSERT') {
                    yield handleOperation('Insert', oldRow, newRow);
                    yield handleSelectorOperation('Insert', oldRow, newRow);
                }
                else if (operation === 'DELETE') {
                    yield handleOperation('Delete', oldRow, newRow);
                    yield handleSelectorOperation('Delete', oldRow, newRow);
                }
                const typeId = current.type_id;
                const handleScheduleId = yield deep.id('@deep-foundation/core', 'HandleSchedule');
                if (typeId === handleScheduleId && (operation === 'INSERT' || operation === 'DELETE')) {
                    yield handleSchedule(current, operation);
                }
                if (typeId === handlePortId && (operation === 'INSERT' || operation === 'DELETE')) {
                    yield handlePort(current, operation);
                }
                log(`Link ${current.id} is proccessed.`);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    log('resolve', current.id);
                    yield (0, promise_1.resolve)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                        Results: false,
                    });
                }
                return res.status(200).json({});
            }
            catch (e) {
                error('error', e, (_j = (_h = e === null || e === void 0 ? void 0 : e.graphQLErrors) === null || _h === void 0 ? void 0 : _h[0]) === null || _j === void 0 ? void 0 : _j.extensions);
                if (operation === 'INSERT' && !global_ids_1.DENIED_IDS.includes(current.type_id) && global_ids_1.ALLOWED_IDS.includes(current.type_id)) {
                    log('reject', current.id);
                    yield (0, promise_1.reject)({
                        id: current.id, client,
                        Then: yield deep.id('@deep-foundation/core', 'Then'),
                        Promise: yield deep.id('@deep-foundation/core', 'Promise'),
                        Resolved: yield deep.id('@deep-foundation/core', 'Resolved'),
                        Rejected: yield deep.id('@deep-foundation/core', 'Rejected'),
                        Results: false,
                    });
                }
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (e) {
        return res.status(500).json({ error: e.toString() });
    }
});
//# sourceMappingURL=links.js.map