"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPromiseResults = exports.deleteScheduleHandler = exports.deletePromiseResult = void 0;
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const chai_1 = require("chai");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const debug_1 = __importDefault(require("debug"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const handlers_1 = require("../imports/handlers");
const debug = (0, debug_1.default)('deeplinks:tests:handlers');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const wait_on_1 = __importDefault(require("wait-on"));
const get_port_1 = __importDefault(require("get-port"));
jest.setTimeout(120000);
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({ apolloClient });
const DELAY = +process.env.DELAY || 0;
const delay = time => new Promise(res => setTimeout(res, time));
let packageWithPermissions;
let lastFreeId = 9999999999;
const nextFreeId = () => {
    lastFreeId -= 1;
    return lastFreeId;
};
let lastHandlerResult = 1;
const nextHandlerResult = () => {
    lastHandlerResult += 1;
    return lastHandlerResult;
};
const collectIds = (data) => {
    const ids = [];
    if (Array.isArray(data)) {
        data.forEach(item => {
            collectIds(item).forEach(id => ids.push(id));
        });
    }
    if (data.id) {
        ids.push(data.id);
    }
    if (data.to) {
        collectIds(data.to).forEach(id => ids.push(id));
    }
    if (data.from) {
        collectIds(data.from).forEach(id => ids.push(id));
    }
    if (data.in) {
        data.in.forEach(item => {
            collectIds(item).forEach(id => ids.push(id));
        });
    }
    if (data.out) {
        data.out.forEach(item => {
            collectIds(item).forEach(id => ids.push(id));
        });
    }
    return ids;
};
const insertPackageWithPermissions = (forcePackageId) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const Rule = yield deep.id('@deep-foundation/core', 'Rule');
    const RuleSubject = yield deep.id('@deep-foundation/core', 'RuleSubject');
    const Selector = yield deep.id('@deep-foundation/core', 'Selector');
    const RuleObject = yield deep.id('@deep-foundation/core', 'RuleObject');
    const RuleAction = yield deep.id('@deep-foundation/core', 'RuleAction');
    const SelectorInclude = yield deep.id('@deep-foundation/core', 'SelectorInclude');
    const SelectorTree = yield deep.id('@deep-foundation/core', 'SelectorTree');
    const AllowSelect = yield deep.id('@deep-foundation/core', 'AllowSelect');
    const AllowInsertType = yield deep.id('@deep-foundation/core', 'AllowInsertType');
    const containTree = yield deep.id('@deep-foundation/core', 'containTree');
    const joinTree = yield deep.id('@deep-foundation/core', 'joinTree');
    let packageId;
    let packageValueId;
    let containId;
    let containValueId;
    let typeId;
    if (forcePackageId) {
        packageId = forcePackageId;
    }
    else {
        const packageTypeId = yield deep.id('@deep-foundation/core', 'Package');
        const containTypeId = yield deep.id('@deep-foundation/core', 'Contain');
        const typeTypeId = yield deep.id('@deep-foundation/core', 'Type');
        const $package = (yield deep.insert({
            type_id: packageTypeId,
            string: { data: { value: '@deep-foundation/test-package' } },
            out: {
                data: {
                    type_id: containTypeId,
                    string: { data: { value: 'test-type' } },
                    to: {
                        data: {
                            type_id: typeTypeId
                        }
                    },
                }
            }
        }, { name: 'INSERT_PACKAGE', returning: `
      id
      string {
        id
      }
      out {
        id
        string {
          id
        }
        to {
          id
        }
      }
    ` })).data[0];
        packageId = $package.id;
        packageValueId = $package.string.id;
        containId = (_b = (_a = $package === null || $package === void 0 ? void 0 : $package.out) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
        containValueId = (_f = (_e = (_d = (_c = $package === null || $package === void 0 ? void 0 : $package.out) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.string) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id;
        typeId = (_j = (_h = (_g = $package === null || $package === void 0 ? void 0 : $package.out) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.to) === null || _j === void 0 ? void 0 : _j.id;
    }
    const rule = yield deep.insert({
        type_id: Rule,
        out: { data: [
                {
                    type_id: RuleSubject,
                    to: { data: {
                            type_id: Selector,
                            out: { data: [
                                    {
                                        type_id: SelectorInclude,
                                        to_id: packageId,
                                        out: { data: {
                                                type_id: SelectorTree,
                                                to_id: joinTree,
                                            }, },
                                    },
                                ] },
                        }, },
                },
                {
                    type_id: RuleObject,
                    to: { data: {
                            type_id: Selector,
                            out: { data: [
                                    {
                                        type_id: SelectorInclude,
                                        to_id: packageId,
                                        out: { data: {
                                                type_id: SelectorTree,
                                                to_id: containTree,
                                            }, },
                                    },
                                ], },
                        }, },
                },
                {
                    type_id: RuleAction,
                    to: { data: {
                            type_id: Selector,
                            out: { data: [
                                    {
                                        type_id: SelectorInclude,
                                        to_id: AllowInsertType,
                                        out: { data: {
                                                type_id: SelectorTree,
                                                to_id: containTree,
                                            }, },
                                    },
                                    {
                                        type_id: SelectorInclude,
                                        to_id: AllowSelect,
                                        out: { data: {
                                                type_id: SelectorTree,
                                                to_id: containTree,
                                            }, },
                                    }
                                ], },
                        }, },
                },
            ], },
    }, { returning: `
    id
    out {
      id
      to {
        id
        out {
          id
          out {
            id
          }
        }
      }
    }
  ` });
    const ids = collectIds(rule.data);
    return { packageId, packageValueId, containId, containValueId, typeId, ruleIds: ids };
});
const deletePackageWithPermissions = ($package) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, handlers_1.deleteId)($package.typeId);
    yield (0, handlers_1.deleteId)($package.containValueId, { table: 'strings' });
    yield (0, handlers_1.deleteId)($package.containId);
    yield (0, handlers_1.deleteId)($package.packageValueId, { table: 'strings' });
    yield (0, handlers_1.deleteId)($package.packageId);
    yield (0, handlers_1.deleteIds)($package.ruleIds, error);
});
const insertOperationHandlerForSchedule = (schedule, code, forceOwnerId) => __awaiter(void 0, void 0, void 0, function* () {
    const syncTextFileTypeId = yield deep.id('@deep-foundation/core', 'SyncTextFile');
    const handlerJSFile = (yield deep.insert({
        type_id: syncTextFileTypeId,
    }, { name: 'INSERT_HANDLER_JS_FILE' })).data[0];
    const handlerJSFileValue = (yield deep.insert({ link_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id, value: code }, { table: 'strings' })).data[0];
    const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
    const isolationProviderThatSupportsJSExecutionProviderId = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
    const handler = (yield deep.insert({
        from_id: isolationProviderThatSupportsJSExecutionProviderId,
        type_id: handlerTypeId,
        to_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
    }, { name: 'INSERT_HANDLER' })).data[0];
    const ownerId = forceOwnerId || (yield deep.id('deep', 'admin'));
    const ownerContainHandler = (yield deep.insert({
        from_id: ownerId,
        type_id: yield deep.id('@deep-foundation/core', 'Contain'),
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    }, { name: 'INSERT_ADMIN_CONTAIN_HANDLER' })).data[0];
    const scheduleTypeId = yield deep.id('@deep-foundation/core', 'Schedule');
    const scheduleNode = (yield deep.insert({
        type_id: scheduleTypeId,
    }, { name: 'INSERT_SCHEDULE' })).data[0];
    const scheduleValue = (yield deep.insert({ link_id: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id, value: schedule }, { table: 'strings' })).data[0];
    const handleScheduleTypeId = yield deep.id('@deep-foundation/core', 'HandleSchedule');
    const handleOperation = (yield deep.insert({
        from_id: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id,
        type_id: handleScheduleTypeId,
        to_id: handler === null || handler === void 0 ? void 0 : handler.id,
    }, { name: 'INSERT_INSERT_HANDLER' })).data[0];
    return {
        handlerId: handler === null || handler === void 0 ? void 0 : handler.id,
        handleOperationId: handleOperation === null || handleOperation === void 0 ? void 0 : handleOperation.id,
        handlerJSFileId: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
        handlerJSFileValueId: handlerJSFileValue === null || handlerJSFileValue === void 0 ? void 0 : handlerJSFileValue.id,
        scheduleId: scheduleNode === null || scheduleNode === void 0 ? void 0 : scheduleNode.id,
        scheduleValueId: scheduleValue === null || scheduleValue === void 0 ? void 0 : scheduleValue.id,
        ownerContainHandlerId: ownerContainHandler === null || ownerContainHandler === void 0 ? void 0 : ownerContainHandler.id,
    };
});
function deletePromiseResult(promiseResult, linkId) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
    return __awaiter(this, void 0, void 0, function* () {
        const resultLinkId = (_b = (_a = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
        const thenLinkId = (_g = (_f = (_e = (_d = (_c = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.from) === null || _e === void 0 ? void 0 : _e.in) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.id;
        const valueId = (_h = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.object) === null || _h === void 0 ? void 0 : _h.id;
        const promiseResultId = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.id;
        const promiseId = (_l = (_k = (_j = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.from) === null || _l === void 0 ? void 0 : _l.id;
        const promiseReasonId = (_q = (_p = (_o = (_m = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.in) === null || _m === void 0 ? void 0 : _m[0]) === null || _o === void 0 ? void 0 : _o.out) === null || _p === void 0 ? void 0 : _p[0]) === null || _q === void 0 ? void 0 : _q.id;
        yield (0, handlers_1.deleteId)(promiseReasonId);
        yield (0, handlers_1.deleteIds)([resultLinkId, thenLinkId], error);
        yield (0, handlers_1.deleteId)(valueId, { table: 'objects' });
        yield (0, handlers_1.deleteIds)([promiseResultId, promiseId, linkId], error);
    });
}
exports.deletePromiseResult = deletePromiseResult;
const deleteScheduleHandler = (handler) => __awaiter(void 0, void 0, void 0, function* () {
    yield (0, handlers_1.deleteHandler)(handler);
    yield (0, handlers_1.deleteId)(handler.scheduleValueId, { table: 'strings' });
    yield (0, handlers_1.deleteId)(handler.scheduleId);
});
exports.deleteScheduleHandler = deleteScheduleHandler;
function getPromiseResults(deep, resultTypeId, linkId) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const promiseTypeId = yield deep.id('@deep-foundation/core', 'Promise');
        const thenTypeId = yield deep.id('@deep-foundation/core', 'Then');
        const promiseReasonTypeId = yield deep.id('@deep-foundation/core', 'PromiseReason');
        const client = deep.apolloClient;
        const queryString = `{
    links(where: { 
      in: {
        type_id: { _eq: ${resultTypeId} }, # Resolved/Rejected
        from: { 
          type_id: { _eq: ${promiseTypeId} }, # Promise
          in: { 
            type_id: { _eq: ${thenTypeId} } # Then
            from_id: { _eq: ${linkId} } # linkId
          }
        }
      },
    }) {
      id
      object {
        id
        value
      }
      in(where: { type_id: { _eq: ${resultTypeId} } }) {
        id
        from {
          id
          in(where: { type_id: { _eq: ${thenTypeId} } }) {
            id
          }
        }
        out(where: { type_id: { _eq: ${promiseReasonTypeId} } }) {
          id
        }
      }
    }
  }`;
        return (_b = (_a = (yield client.query({
            query: (0, graphql_tag_1.default) `${queryString}`,
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.links;
    });
}
exports.getPromiseResults = getPromiseResults;
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    packageWithPermissions = yield insertPackageWithPermissions();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    deletePackageWithPermissions(packageWithPermissions);
}));
describe.skip('sync function handle by type with resolve', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle update when value is inserted`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        let promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle update when value is updated`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        yield deep.update({ link_id: linkId }, { value: numberToReturn + 1 }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const matchedPromiseResults = promiseResults.filter(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const promiseResult of matchedPromiseResults) {
            yield deletePromiseResult(promiseResult);
        }
        yield (0, handlers_1.deleteId)(linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!matchedPromiseResults);
        chai_1.assert.equal(matchedPromiseResults.length, 2);
    }));
    it(`handle update when value is deleted`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        yield deep.delete({ link_id: { _eq: linkId } }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const matchedPromiseResults = promiseResults.filter(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const promiseResult of matchedPromiseResults) {
            yield deletePromiseResult(promiseResult);
        }
        yield (0, handlers_1.deleteId)(linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!matchedPromiseResults);
        chai_1.assert.equal(matchedPromiseResults.length, 2);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, typeId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield (0, handlers_1.deleteId)(linkId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe.skip('sync function handle by type with reject', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `(arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, typeId, `(arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield (0, handlers_1.deleteId)(linkId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('async function handle by type with reject', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `async (arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToThrow = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, typeId, `async (arg) => { throw ${numberToThrow}; return { "error": "return is not possible" }; }`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield (0, handlers_1.deleteId)(linkId);
        yield deep.await(linkId);
        const rejectedTypeId = yield deep.id('@deep-foundation/core', 'Rejected');
        const promiseResults = yield getPromiseResults(deep, rejectedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a; return ((_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === numberToThrow; });
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('sync function handle by schedule with resolve', () => {
    it(`handle schedule`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const handler = yield insertOperationHandlerForSchedule('* * * * *', `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        yield deep.await(handler.scheduleId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, handler.scheduleId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        yield deletePromiseResult(promiseResult, handler.scheduleId);
        yield (0, exports.deleteScheduleHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('async function handle by type with resolve using deep client', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const queryTypeId = yield deep.id('@deep-foundation/core', 'Query');
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `async (arg) => {
       const deep = arg.deep;
       const queryTypeId = await deep.id('@deep-foundation/core', 'Query');
       const queryId = (await deep.insert({ type_id: queryTypeId }))?.data?.[0]?.id;
      //  const queryId = (await deep.insert({ type_id: ${queryTypeId} }))?.data?.[0]?.id;
       return { queryId, result: ${numberToReturn}}
    }`);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        const queryId = (_b = (_a = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.queryId;
        const query = (yield deep.select({ id: { _eq: queryId } })).data[0];
        yield (0, handlers_1.deleteId)(queryId);
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.equal(query.id, queryId);
        chai_1.assert.isTrue(!!promiseResult);
    }));
    it(`handle insert with package jwt`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _c, _d;
        const numberToReturn = nextHandlerResult();
        const typeId = yield deep.id('@deep-foundation/core', 'Type');
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `async (arg) => {
       const deep = arg.deep;
       const nodeTypeId = await deep.id('@deep-foundation/test-package', 'test-type');
       const nodeId = (await deep.insert({ type_id: nodeTypeId }))?.data?.[0]?.id;
       return { nodeId, result: ${numberToReturn}}
    }`, packageWithPermissions.packageId);
        const linkId = yield (0, handlers_1.ensureLinkIsCreated)(typeId);
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const promiseResult = promiseResults.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        const nodeId = (_d = (_c = promiseResult === null || promiseResult === void 0 ? void 0 : promiseResult.object) === null || _c === void 0 ? void 0 : _c.value) === null || _d === void 0 ? void 0 : _d.nodeId;
        const node = (yield deep.select({ id: { _eq: nodeId } })).data[0];
        yield (0, handlers_1.deleteId)(nodeId);
        yield deletePromiseResult(promiseResult, linkId);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.equal(node.id, nodeId);
        chai_1.assert.isTrue(!!promiseResult);
    }));
});
describe('handle port', () => {
    it(`handle port`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f;
        const port = yield (0, get_port_1.default)();
        const portTypeId = yield deep.id('@deep-foundation/core', 'Port');
        const portId = (_c = (_b = (_a = (yield deep.insert({
            type_id: portTypeId,
            number: { data: { value: port } }
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
        const jsDockerIsolationProviderId = yield deep.id('@deep-foundation/core', 'JSDockerIsolationProvider');
        const handlePortTypeId = yield deep.id('@deep-foundation/core', 'HandlePort');
        const hanlePortLinkId = (_f = (_e = (_d = (yield deep.insert({ from_id: portId, type_id: handlePortTypeId, to_id: jsDockerIsolationProviderId }))) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id;
        yield deep.await(hanlePortLinkId);
        log("waiting for container to be created");
        yield (0, wait_on_1.default)({ resources: [`http://localhost:${port}/healthz`] });
        log("container is up");
        yield (0, handlers_1.deleteId)(hanlePortLinkId);
        log("waiting for container to be removed");
        yield (0, wait_on_1.default)({
            resources: [
                `http://localhost:${port}/healthz`
            ],
            reverse: true,
        });
        log("container is down");
    }));
});
describe('handle route', () => {
    it(`handle route`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y;
        const port = 4005;
        const portTypeId = yield deep.id('@deep-foundation/core', 'Port');
        const portId = (_c = (_b = (_a = (yield deep.insert({
            type_id: portTypeId,
        }))) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
        const portValue = (_e = (_d = (yield deep.insert({
            link_id: portId,
            value: port
        }, { table: 'numbers' }))) === null || _d === void 0 ? void 0 : _d.data) === null || _e === void 0 ? void 0 : _e[0];
        const routeTypeId = yield deep.id('@deep-foundation/core', 'Route');
        const routeId = (_h = (_g = (_f = (yield deep.insert({
            type_id: routeTypeId,
        }))) === null || _f === void 0 ? void 0 : _f.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.id;
        const routerTypeId = yield deep.id('@deep-foundation/core', 'Router');
        const routerId = (_l = (_k = (_j = (yield deep.insert({
            type_id: routerTypeId,
        }))) === null || _j === void 0 ? void 0 : _j.data) === null || _k === void 0 ? void 0 : _k[0]) === null || _l === void 0 ? void 0 : _l.id;
        const routerListeningTypeId = yield deep.id('@deep-foundation/core', 'RouterListening');
        const routerListeningId = (_p = (_o = (_m = (yield deep.insert({
            type_id: routerListeningTypeId,
            from_id: routerId,
            to_id: portId,
        }))) === null || _m === void 0 ? void 0 : _m.data) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.id;
        const route = '/passport';
        const routerStringUseTypeId = yield deep.id('@deep-foundation/core', 'RouterStringUse');
        const routerStringUseId = (_s = (_r = (_q = (yield deep.insert({
            type_id: routerStringUseTypeId,
            to_id: routerId,
            from_id: routeId,
            string: { data: { value: route } }
        }))) === null || _q === void 0 ? void 0 : _q.data) === null || _r === void 0 ? void 0 : _r[0]) === null || _s === void 0 ? void 0 : _s.id;
        const syncTextFileTypeId = yield deep.id('@deep-foundation/core', 'SyncTextFile');
        const handlerJSFile = (yield deep.insert({
            type_id: syncTextFileTypeId,
        }, { name: 'INSERT_HANDLER_JS_FILE' })).data[0];
        const handlerJSFileValue = (yield deep.insert({ link_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id, value: `async (req, res) => { res.send('ok'); }` }, { table: 'strings' })).data[0];
        const isolationProviderThatSupportsJSExecutionProviderId = yield deep.id('@deep-foundation/core', 'dockerSupportsJs');
        const handlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
        const handlerId = (_v = (_u = (_t = (yield deep.insert({
            type_id: handlerTypeId,
            from_id: isolationProviderThatSupportsJSExecutionProviderId,
            to_id: handlerJSFile === null || handlerJSFile === void 0 ? void 0 : handlerJSFile.id,
        }))) === null || _t === void 0 ? void 0 : _t.data) === null || _u === void 0 ? void 0 : _u[0]) === null || _v === void 0 ? void 0 : _v.id;
        const containTypeId = yield deep.id('@deep-foundation/core', 'Contain');
        const ownerId = yield deep.id('deep', 'admin');
        const ownerContainHandler = (yield deep.insert({
            from_id: ownerId,
            type_id: containTypeId,
            to_id: handlerId,
        }, { name: 'INSERT_ADMIN_CONTAIN_HANDLER' })).data[0];
        const handleRouteTypeId = yield deep.id('@deep-foundation/core', 'HandleRoute');
        const handleRouteLinkId = (_y = (_x = (_w = (yield deep.insert({
            from_id: routeId,
            type_id: handleRouteTypeId,
            to_id: handlerId,
        }))) === null || _w === void 0 ? void 0 : _w.data) === null || _x === void 0 ? void 0 : _x[0]) === null || _y === void 0 ? void 0 : _y.id;
        const url = `http://localhost:${port}${route}`;
        log("waiting for route to be created");
        yield (0, wait_on_1.default)({ resources: [url] });
        log("route handler is up");
        const response = yield (0, node_fetch_1.default)(url);
        const text = yield response.text();
        chai_1.assert.equal(text, 'ok');
        yield (0, handlers_1.deleteId)(handleRouteLinkId);
        yield (0, handlers_1.deleteId)(ownerContainHandler.id);
        yield (0, handlers_1.deleteId)(handlerId);
        yield (0, handlers_1.deleteId)(handlerJSFileValue.id, { table: 'strings' });
        yield (0, handlers_1.deleteId)(handlerJSFile.id);
        yield (0, handlers_1.deleteId)(routerStringUseId);
        yield (0, handlers_1.deleteId)(routerListeningId);
        yield (0, handlers_1.deleteId)(routerId);
        yield (0, handlers_1.deleteId)(routeId);
        yield (0, handlers_1.deleteId)(portValue.id, { table: 'numbers' });
        yield (0, handlers_1.deleteId)(portId);
        log("waiting for route to be deleted");
        yield (0, wait_on_1.default)({ resources: [url], reverse: true });
        log("route handler is down");
    }));
    it(`handle route hierarchical insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        var _z, _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13, _14, _15, _16, _17, _18, _19, _20, _21, _22, _23, _24, _25, _26, _27, _28, _29, _30, _31, _32, _33, _34, _35, _36, _37, _38, _39, _40, _41, _42, _43, _44, _45, _46, _47, _48, _49, _50, _51, _52, _53, _54, _55, _56, _57, _58, _59, _60, _61, _62, _63, _64, _65, _66, _67, _68, _69, _70, _71, _72, _73, _74, _75, _76, _77, _78, _79, _80, _81, _82, _83, _84;
        const port = 4001;
        const route = '/passport';
        const insertResult = yield deep.insert({
            type_id: yield deep.id('@deep-foundation/core', 'Port'),
            number: { data: { value: port } },
            in: { data: {
                    type_id: yield deep.id('@deep-foundation/core', 'RouterListening'),
                    from: { data: {
                            type_id: yield deep.id('@deep-foundation/core', 'Router'),
                            in: { data: {
                                    type_id: yield deep.id('@deep-foundation/core', 'RouterStringUse'),
                                    string: { data: { value: route } },
                                    from: { data: {
                                            type_id: yield deep.id('@deep-foundation/core', 'Route'),
                                            out: { data: {
                                                    type_id: yield deep.id('@deep-foundation/core', 'HandleRoute'),
                                                    to: { data: {
                                                            type_id: yield deep.id('@deep-foundation/core', 'Handler'),
                                                            from_id: yield deep.id('@deep-foundation/core', 'dockerSupportsJs'),
                                                            in: { data: {
                                                                    type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                                                                    from_id: yield deep.id('deep', 'admin'),
                                                                    string: { data: { value: 'passport' } },
                                                                } },
                                                            to: { data: {
                                                                    type_id: yield deep.id('@deep-foundation/core', 'SyncTextFile'),
                                                                    string: { data: {
                                                                            value: `async (req, res) => { res.send('ok'); }`,
                                                                        } },
                                                                } },
                                                        } },
                                                } },
                                        } },
                                } },
                        } },
                } },
        }, {
            returning: `
        id
        number {
          id
        }
        in {
          id
          from {
            id
            in {
              id
              from {
                id
                out {
                  id
                  to {
                    id
                    in {
                      id
                      string {
                        id
                      }
                    }
                    to {
                      id
                      string {
                        id
                      }
                    }
                  }
                }
              }
            }
          }
        }
      `,
            name: 'INSERT_HANDLE_ROUTE_HIERARCHICAL',
        });
        const portId = (_0 = (_z = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _z === void 0 ? void 0 : _z[0]) === null || _0 === void 0 ? void 0 : _0.id;
        const portValueId = (_3 = (_2 = (_1 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _1 === void 0 ? void 0 : _1[0]) === null || _2 === void 0 ? void 0 : _2.number) === null || _3 === void 0 ? void 0 : _3.id;
        const routerListeningId = (_7 = (_6 = (_5 = (_4 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _4 === void 0 ? void 0 : _4[0]) === null || _5 === void 0 ? void 0 : _5.in) === null || _6 === void 0 ? void 0 : _6[0]) === null || _7 === void 0 ? void 0 : _7.id;
        const routerId = (_12 = (_11 = (_10 = (_9 = (_8 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _8 === void 0 ? void 0 : _8[0]) === null || _9 === void 0 ? void 0 : _9.in) === null || _10 === void 0 ? void 0 : _10[0]) === null || _11 === void 0 ? void 0 : _11.from) === null || _12 === void 0 ? void 0 : _12.id;
        const routerStringUseId = (_19 = (_18 = (_17 = (_16 = (_15 = (_14 = (_13 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _13 === void 0 ? void 0 : _13[0]) === null || _14 === void 0 ? void 0 : _14.in) === null || _15 === void 0 ? void 0 : _15[0]) === null || _16 === void 0 ? void 0 : _16.from) === null || _17 === void 0 ? void 0 : _17.in) === null || _18 === void 0 ? void 0 : _18[0]) === null || _19 === void 0 ? void 0 : _19.id;
        const routeId = (_27 = (_26 = (_25 = (_24 = (_23 = (_22 = (_21 = (_20 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _20 === void 0 ? void 0 : _20[0]) === null || _21 === void 0 ? void 0 : _21.in) === null || _22 === void 0 ? void 0 : _22[0]) === null || _23 === void 0 ? void 0 : _23.from) === null || _24 === void 0 ? void 0 : _24.in) === null || _25 === void 0 ? void 0 : _25[0]) === null || _26 === void 0 ? void 0 : _26.from) === null || _27 === void 0 ? void 0 : _27.id;
        const handleRouteId = (_37 = (_36 = (_35 = (_34 = (_33 = (_32 = (_31 = (_30 = (_29 = (_28 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _28 === void 0 ? void 0 : _28[0]) === null || _29 === void 0 ? void 0 : _29.in) === null || _30 === void 0 ? void 0 : _30[0]) === null || _31 === void 0 ? void 0 : _31.from) === null || _32 === void 0 ? void 0 : _32.in) === null || _33 === void 0 ? void 0 : _33[0]) === null || _34 === void 0 ? void 0 : _34.from) === null || _35 === void 0 ? void 0 : _35.out) === null || _36 === void 0 ? void 0 : _36[0]) === null || _37 === void 0 ? void 0 : _37.id;
        const handlerId = (_48 = (_47 = (_46 = (_45 = (_44 = (_43 = (_42 = (_41 = (_40 = (_39 = (_38 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _38 === void 0 ? void 0 : _38[0]) === null || _39 === void 0 ? void 0 : _39.in) === null || _40 === void 0 ? void 0 : _40[0]) === null || _41 === void 0 ? void 0 : _41.from) === null || _42 === void 0 ? void 0 : _42.in) === null || _43 === void 0 ? void 0 : _43[0]) === null || _44 === void 0 ? void 0 : _44.from) === null || _45 === void 0 ? void 0 : _45.out) === null || _46 === void 0 ? void 0 : _46[0]) === null || _47 === void 0 ? void 0 : _47.to) === null || _48 === void 0 ? void 0 : _48.id;
        const handlerJSFileId = (_59 = (_58 = (_57 = (_56 = (_55 = (_54 = (_53 = (_52 = (_51 = (_50 = (_49 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _49 === void 0 ? void 0 : _49[0]) === null || _50 === void 0 ? void 0 : _50.in) === null || _51 === void 0 ? void 0 : _51[0]) === null || _52 === void 0 ? void 0 : _52.from) === null || _53 === void 0 ? void 0 : _53.in) === null || _54 === void 0 ? void 0 : _54[0]) === null || _55 === void 0 ? void 0 : _55.from) === null || _56 === void 0 ? void 0 : _56.out) === null || _57 === void 0 ? void 0 : _57[0]) === null || _58 === void 0 ? void 0 : _58.to) === null || _59 === void 0 ? void 0 : _59.id;
        const handlerJSFileValueId = (_71 = (_70 = (_69 = (_68 = (_67 = (_66 = (_65 = (_64 = (_63 = (_62 = (_61 = (_60 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _60 === void 0 ? void 0 : _60[0]) === null || _61 === void 0 ? void 0 : _61.in) === null || _62 === void 0 ? void 0 : _62[0]) === null || _63 === void 0 ? void 0 : _63.from) === null || _64 === void 0 ? void 0 : _64.in) === null || _65 === void 0 ? void 0 : _65[0]) === null || _66 === void 0 ? void 0 : _66.from) === null || _67 === void 0 ? void 0 : _67.out) === null || _68 === void 0 ? void 0 : _68[0]) === null || _69 === void 0 ? void 0 : _69.to) === null || _70 === void 0 ? void 0 : _70.string) === null || _71 === void 0 ? void 0 : _71.id;
        const ownerContainHandlerId = (_84 = (_83 = (_82 = (_81 = (_80 = (_79 = (_78 = (_77 = (_76 = (_75 = (_74 = (_73 = (_72 = insertResult === null || insertResult === void 0 ? void 0 : insertResult.data) === null || _72 === void 0 ? void 0 : _72[0]) === null || _73 === void 0 ? void 0 : _73.in) === null || _74 === void 0 ? void 0 : _74[0]) === null || _75 === void 0 ? void 0 : _75.from) === null || _76 === void 0 ? void 0 : _76.in) === null || _77 === void 0 ? void 0 : _77[0]) === null || _78 === void 0 ? void 0 : _78.from) === null || _79 === void 0 ? void 0 : _79.out) === null || _80 === void 0 ? void 0 : _80[0]) === null || _81 === void 0 ? void 0 : _81.to) === null || _82 === void 0 ? void 0 : _82.in) === null || _83 === void 0 ? void 0 : _83[0]) === null || _84 === void 0 ? void 0 : _84.id;
        const url = `http://localhost:${port}${route}`;
        log("waiting for route to be created");
        yield (0, wait_on_1.default)({ resources: [url] });
        log("route handler is up");
        const response = yield (0, node_fetch_1.default)(url);
        const text = yield response.text();
        chai_1.assert.equal(text, 'ok');
        yield (0, handlers_1.deleteId)(handleRouteId);
        yield (0, handlers_1.deleteId)(ownerContainHandlerId);
        yield (0, handlers_1.deleteId)(handlerId);
        yield (0, handlers_1.deleteId)(handlerJSFileValueId, { table: 'strings' });
        yield (0, handlers_1.deleteId)(handlerJSFileId);
        yield (0, handlers_1.deleteId)(routerStringUseId);
        yield (0, handlers_1.deleteId)(routerListeningId);
        yield (0, handlers_1.deleteId)(routerId);
        yield (0, handlers_1.deleteId)(routeId);
        yield (0, handlers_1.deleteId)(portValueId, { table: 'numbers' });
        yield (0, handlers_1.deleteId)(portId);
        log("waiting for route to be deleted");
        yield (0, wait_on_1.default)({ resources: [url], reverse: true });
        log("route handler is down");
    }));
});
describe('handle by selector', () => {
    it(`handle insert`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
        const selector = yield (0, handlers_1.insertSelector)();
        const { nodeTypeId, linkTypeId, treeId, treeIncludesIds, selectorId, selectorIncludeId, selectorTreeId, rootId } = selector;
        const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, selectorId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const selectorItems = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
        log(`awaiting ${selectorItems[1].linkId} link.`);
        yield deep.await(selectorItems[1].linkId);
        log(`awaiting ${selectorItems[0].linkId} link.`);
        yield deep.await(selectorItems[0].linkId);
        const resolvedTypeId1 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults1 = yield getPromiseResults(deep, resolvedTypeId1, selectorItems[1].linkId);
        const promiseResult1 = promiseResults1.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        const resolvedTypeId2 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults2 = yield getPromiseResults(deep, resolvedTypeId2, selectorItems[0].linkId);
        const promiseResult2 = promiseResults2.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const selectorItem of selectorItems) {
            yield (0, handlers_1.deleteId)(selectorItem.linkId);
            yield (0, handlers_1.deleteId)(selectorItem.nodeId);
        }
        yield (0, handlers_1.deleteSelector)(selector);
        yield deletePromiseResult(promiseResult1);
        yield deletePromiseResult(promiseResult2);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult1);
        chai_1.assert.isTrue(!!promiseResult2);
    }));
    it(`handle delete`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
        const selector = yield (0, handlers_1.insertSelector)();
        const { nodeTypeId, linkTypeId, treeId, treeIncludesIds, selectorId, selectorIncludeId, selectorTreeId, rootId } = selector;
        const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, selectorId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const selectorItems = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
        yield (0, handlers_1.deleteId)(selectorItems[1].linkId);
        yield (0, handlers_1.deleteId)(selectorItems[0].linkId);
        yield deep.await(selectorItems[1].linkId);
        yield deep.await(selectorItems[0].linkId);
        const resolvedTypeId1 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults1 = yield getPromiseResults(deep, resolvedTypeId1, selectorItems[1].linkId);
        const promiseResult1 = promiseResults1.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        const resolvedTypeId2 = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults2 = yield getPromiseResults(deep, resolvedTypeId2, selectorItems[0].linkId);
        const promiseResult2 = promiseResults2.find(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const selectorItem of selectorItems) {
            yield (0, handlers_1.deleteId)(selectorItem.linkId);
            yield (0, handlers_1.deleteId)(selectorItem.nodeId);
        }
        yield (0, handlers_1.deleteSelector)(selector);
        yield deletePromiseResult(promiseResult1);
        yield deletePromiseResult(promiseResult2);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!promiseResult1);
        chai_1.assert.isTrue(!!promiseResult2);
    }));
    it(`handle update when value is updated`, () => __awaiter(void 0, void 0, void 0, function* () {
        const numberToReturn = nextHandlerResult();
        const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
        const selector = yield (0, handlers_1.insertSelector)();
        const { nodeTypeId, linkTypeId, treeId, treeIncludesIds, selectorId, selectorIncludeId, selectorTreeId, rootId } = selector;
        const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, selectorId, `(arg) => {console.log(arg); return {result: ${numberToReturn}}}`);
        const selectorItems = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
        const linkId = selectorItems[1].linkId;
        yield deep.insert({ link_id: linkId, value: numberToReturn }, { table: 'numbers' });
        yield deep.await(linkId);
        yield deep.update({ link_id: linkId }, { value: numberToReturn + 1 }, { table: 'numbers' });
        yield deep.await(linkId);
        const resolvedTypeId = yield deep.id('@deep-foundation/core', 'Resolved');
        const promiseResults = yield getPromiseResults(deep, resolvedTypeId, linkId);
        const matchedPromiseResults = promiseResults.filter(link => { var _a, _b; return ((_b = (_a = link.object) === null || _a === void 0 ? void 0 : _a.value) === null || _b === void 0 ? void 0 : _b.result) === numberToReturn; });
        for (const promiseResult of matchedPromiseResults) {
            yield deletePromiseResult(promiseResult);
        }
        for (const selectorItem of selectorItems) {
            yield (0, handlers_1.deleteId)(selectorItem.linkId);
            yield (0, handlers_1.deleteId)(selectorItem.nodeId);
        }
        yield (0, handlers_1.deleteId)(linkId);
        yield (0, handlers_1.deleteSelector)(selector);
        yield (0, handlers_1.deleteHandler)(handler);
        chai_1.assert.isTrue(!!matchedPromiseResults);
        chai_1.assert.equal(matchedPromiseResults.length, 2);
    }));
});
//# sourceMappingURL=handlers.js.map